@@ -39,6 +39,7 @@ function(x, start, densfun, distnfun, ..., distn, mlen = 1, largest = TRUE,
     opt <- optim(start, nllh, x = x, hessian = TRUE, ...,
                  densfun = densfun, distnfun = distnfun, mlen = mlen,
                  largest = largest, method = method)
+	if(is.null(names(opt$par))) names(opt$par) <- nm
     if (opt$convergence != 0) {
         warning("optimization may not have succeeded")
         if(opt$convergence == 1) opt$convergence <- "iteration limit reached"
@@ -110,6 +111,7 @@ function(x, start, densfun, distnfun, ..., distn, mlen = 1, j = 1,
     opt <- optim(start, nllh, x = x, hessian = TRUE, ..., densfun = densfun,
                  distnfun = distnfun, mlen = mlen, j = j, largest = largest,
                  method = method)
+	if(is.null(names(opt$par))) names(opt$par) <- nm
     if (opt$convergence != 0) {
         warning("optimization may not have succeeded")
         if(opt$convergence == 1) opt$convergence <- "iteration limit reached"
@@ -175,9 +177,8 @@ function(x, start, ..., nsloc = NULL, std.err = TRUE, corr = FALSE, method = "BF
             loc <- drop(nslocmat %*% ns)
         }
         else loc <- rep(loc, length.out = length(x))
-        .C("nlgev",
-            x, n, loc, scale, shape, dns = double(1),
-            PACKAGE = "evd")$dns
+        .C(C_nlgev,
+            x, n, loc, scale, shape, dns = double(1))$dns
     }
     if(!is.null(nsloc)) {
         if(is.vector(nsloc)) nsloc <- data.frame(trend = nsloc)
@@ -220,6 +221,7 @@ function(x, start, ..., nsloc = NULL, std.err = TRUE, corr = FALSE, method = "BF
     if(warn.inf && do.call("nllh", start.arg) == 1e6)
         warning("negative log-likelihood is infinite at starting values")
     opt <- optim(start, nllh, hessian = TRUE, ..., method = method)
+	if(is.null(names(opt$par))) names(opt$par) <- nm
     if (opt$convergence != 0) {
         warning("optimization may not have succeeded")
         if(opt$convergence == 1) opt$convergence <- "iteration limit reached"
@@ -278,10 +280,9 @@ function(x, start, ..., nsloc = NULL, prob, std.err = TRUE, corr = FALSE, method
             loc <- drop(nslocmat %*% ns) + loc
         }
         if(any(is.infinite(loc))) return(1e6)
-        .C("nlgev",
+        .C(C_nlgev,
             x, n,
-            loc, scale, shape, dns = double(1),
-            PACKAGE = "evd")$dns
+            loc, scale, shape, dns = double(1))$dns
     }
     if(is.null(nsloc)) loc.param <- "quantile"
     else loc.param <- c("quantile", paste("loc", names(nsloc), sep=""))
@@ -336,6 +337,7 @@ function(x, start, ..., nsloc = NULL, prob, std.err = TRUE, corr = FALSE, method
     if(warn.inf && do.call("nllh", start.arg) == 1e6)
         warning("negative log-likelihood is infinite at starting values")
     opt <- optim(start, nllh, hessian = TRUE, ..., method = method)
+	if(is.null(names(opt$par))) names(opt$par) <- nm
     if (opt$convergence != 0) {
         warning("optimization may not have succeeded")
         if(opt$convergence == 1) opt$convergence <- "iteration limit reached"
@@ -382,6 +384,138 @@ function(x, start, ..., nsloc = NULL, prob, std.err = TRUE, corr = FALSE, method
         n = length(x), prob = prob, loc = loc)
 }
 
+"fgumbel"<-
+function(x, start, ..., nsloc = NULL, prob = NULL, std.err = TRUE,
+    corr = FALSE, method = "BFGS", warn.inf = TRUE)
+{
+    fgev(x = x, ..., shape = 0, nsloc = nsloc, prob = prob, std.err = std.err, 
+      corr = corr, method = method, warn.inf = warn.inf)
+}
+
+"fgumbelx"<-
+function(x, start, ..., nsloc1 = NULL, nsloc2 = NULL, std.err = TRUE,
+    corr = FALSE, method = "BFGS", warn.inf = TRUE)
+{
+    call <- match.call()
+    if(missing(x) || length(x) == 0 || !is.numeric(x)) 
+      stop("`x' must be a non-empty numeric vector")
+  
+    nlgumbelx <- function(loc1, scale1, loc2, scale2)
+    { 
+        if(scale1 <= 0 || scale2 <= 0) return(1e6)
+		if(!is.null(nsloc1)) {
+            ns <- numeric(length(loc.param1))
+            for(i in 1:length(ns))
+                ns[i] <- get(loc.param1[i])
+            loc1 <- drop(nslocmat1 %*% ns)
+        }
+        else loc1 <- rep(loc1, length.out = length(x))
+        if(!is.null(nsloc2)) {
+            ns <- numeric(length(loc.param2))
+            for(i in 1:length(ns))
+                ns[i] <- get(loc.param2[i])
+            loc2 <- drop(nslocmat2 %*% ns)
+        }
+        else loc2 <- rep(loc2, length.out = length(x))
+		
+        if(any(loc1 > loc2)) return(1e6)
+		
+		Ftimesf <- function(x, loc1, scale1, loc2, scale2) {
+          exp(-exp(-(x - loc1)/scale1) + log(1/scale2) - (x - loc2)/scale2 - exp(-(x - loc2)/scale2))
+        }
+        -sum(log(Ftimesf(x = x, loc1 = loc1, scale1 = scale1, loc2 = loc2, scale2 = scale2) + 
+		  Ftimesf(x = x, loc1 = loc2, scale1 = scale2, loc2 = loc1, scale2 = scale1)))
+    }
+	if(!is.null(nsloc1)) {
+        if(is.vector(nsloc1)) nsloc1 <- data.frame(trend = nsloc1)
+        if(nrow(nsloc1) != length(x))
+          stop("`nsloc1' and data are not compatible")
+        nslocmat1 <- cbind(1, as.matrix(nsloc1))
+    }
+    if(!is.null(nsloc2)) {
+        if(is.vector(nsloc2)) nsloc2 <- data.frame(trend = nsloc2)
+        if(nrow(nsloc2) != length(x))
+          stop("`nsloc2' and data are not compatible")
+        nslocmat2 <- cbind(1,as.matrix(nsloc2))
+    }
+    x <- as.double(x[!is.na(x)])
+    n <- as.integer(length(x))
+	loc.param1 <- paste("loc1", c("",names(nsloc1)), sep="")
+    loc.param2 <- paste("loc2", c("",names(nsloc2)), sep="")
+    param <- c(loc.param1, "scale1", loc.param2, "scale2")
+    if(missing(start)) {
+        start <- as.list(numeric(length(param)))
+        names(start) <- param
+        emc <- -digamma(1)
+        b0 <- mean(x)
+        b1 <- sum((1:n-1)/(n-1) * sort(x))/n
+        start$scale1 <- (2*b1-b0)/log(2)
+        start$loc1 <- b0 - start$scale1*emc - start$scale1*log(2)
+		start$scale2 <- start$scale1
+		start$loc2 <- start$loc1 + 1e-02
+        start <- start[!(param %in% names(list(...)))]
+    }
+    if(!is.list(start)) 
+        stop("`start' must be a named list")
+    if(!length(start))
+        stop("there are no parameters left to maximize over")
+    nm <- names(start)
+    l <- length(nm)
+	f <- c(as.list(numeric(length(loc.param1))), formals(nlgumbelx)[2],
+      as.list(numeric(length(loc.param2))), formals(nlgumbelx)[4])
+	  
+    names(f) <- param
+    m <- match(nm, param)
+    if(any(is.na(m))) 
+        stop("`start' specifies unknown arguments")    
+    formals(nlgumbelx) <- c(f[m], f[-m])
+    nllh <- function(p, ...) nlgumbelx(p, ...)
+    if(l > 1)
+        body(nllh) <- parse(text = paste("nlgumbelx(", paste("p[",1:l,
+            "]", collapse = ", "), ", ...)"))
+    fixed.param <- list(...)[names(list(...)) %in% param]
+    if(any(!(param %in% c(nm,names(fixed.param)))))
+        stop("unspecified parameters")
+    start.arg <- c(list(p = unlist(start)), fixed.param)
+    if(warn.inf && do.call("nllh", start.arg) == 1e6)
+        warning("negative log-likelihood is infinite at starting values")
+    opt <- optim(start, nllh, hessian = TRUE, ..., method = method)
+	if(is.null(names(opt$par))) names(opt$par) <- nm
+    if (opt$convergence != 0) {
+        warning("optimization may not have succeeded")
+        if(opt$convergence == 1) opt$convergence <- "iteration limit reached"
+    }
+    else opt$convergence <- "successful"
+    if(std.err) {
+        tol <- .Machine$double.eps^0.5
+        var.cov <- qr(opt$hessian, tol = tol)
+        if(var.cov$rank != ncol(var.cov$qr)) 
+            stop("observed information matrix is singular; use std.err = FALSE")
+        var.cov <- solve(var.cov, tol = tol)
+        std.err <- diag(var.cov)
+        if(any(std.err <= 0))
+            stop("observed information matrix is singular; use std.err = FALSE")
+        std.err <- sqrt(std.err)
+        names(std.err) <- nm
+        if(corr) {
+            .mat <- diag(1/std.err, nrow = length(std.err))
+            corr <- structure(.mat %*% var.cov %*% .mat, dimnames = list(nm,nm))
+            diag(corr) <- rep(1, length(std.err))
+        }
+        else corr <- NULL
+    }
+    else std.err <- var.cov <- corr <- NULL
+    param <- c(opt$par, unlist(fixed.param))
+    ft <- list(estimate = opt$par, std.err = std.err,
+        fixed = unlist(fixed.param), param = param,
+        deviance = 2*opt$value, corr = corr, var.cov = var.cov,
+        convergence = opt$convergence, counts = opt$counts,
+        message = opt$message,
+        data = x, nsloc1 = nsloc1, nsloc2 = nsloc2,
+        n = length(x))
+    structure(c(ft, call = call), class = c("gumbelx", "evd"))
+}
+
 "fpot"<-
 function(x, threshold, model = c("gpd", "pp"), start, npp = length(x), cmax = FALSE, r = 1, ulow = -Inf, rlow = 1, mper = NULL, ..., std.err = TRUE, corr = FALSE, method = "BFGS", warn.inf = TRUE)
 {
@@ -414,18 +548,17 @@ function(x, threshold, model, start, npp = length(x), cmax = FALSE, r = 1, ulow
 {
     if(model == "gpd") {
       nlpot <- function(loc, scale, shape) { 
-        .C("nlgpd",
-            exceed, nhigh, threshold, scale, shape, dns = double(1),
-            PACKAGE = "evd")$dns
+        .C(C_nlgpd,
+            exceed, nhigh, threshold, scale, shape, dns = double(1))$dns
       }
       # Avoids note produced by R CMD check
       formals(nlpot) <- formals(nlpot)[2:3] 
     }
     if(model == "pp") {
       nlpot <- function(loc, scale, shape) {
-        .C("nlpp",
+        .C(C_nlpp,
             exceed, nhigh, loc, scale, shape, threshold, nop,
-            dns = double(1), PACKAGE = "evd")$dns
+            dns = double(1))$dns
       }
     }
     nn <- length(x)
@@ -483,6 +616,7 @@ function(x, threshold, model, start, npp = length(x), cmax = FALSE, r = 1, ulow
     if(warn.inf && do.call("nllh", start.arg) == 1e6)
         warning("negative log-likelihood is infinite at starting values")
     opt <- optim(start, nllh, hessian = TRUE, ..., method = method)
+	if(is.null(names(opt$par))) names(opt$par) <- nm
     if (opt$convergence != 0) {
         warning("optimization may not have succeeded")
         if(opt$convergence == 1) opt$convergence <- "iteration limit reached"
@@ -515,7 +649,7 @@ function(x, threshold, model, start, npp = length(x), cmax = FALSE, r = 1, ulow
     list(estimate = opt$par, std.err = std.err, fixed = unlist(fixed.param),
         param = param, deviance = 2*opt$value, corr = corr, var.cov = var.cov,
         convergence = opt$convergence, counts = opt$counts, message = opt$message,
-        threshold = threshold, r = r, ulow = ulow, rlow = rlow, npp = npp,
+        threshold = threshold, cmax = cmax, r = r, ulow = ulow, rlow = rlow, npp = npp,
         nhigh = nhigh, nat = nat, pat = pat, extind = extind,
         data = x, exceedances = exceed, mper = NULL, scale = scale)
 }
@@ -529,9 +663,8 @@ function(x, threshold, start, npp = length(x), cmax = FALSE, r = 1, ulow = -Inf,
         rlevel <- rlevel - threshold
         if(shape == 0) scale <- rlevel / log(adjmper)
         else scale <- shape * rlevel / (adjmper^shape - 1)
-        .C("nlgpd",
-            exceed, nhigh, threshold, scale, shape, dns = double(1),
-            PACKAGE = "evd")$dns
+        .C(C_nlgpd,
+            exceed, nhigh, threshold, scale, shape, dns = double(1))$dns
     }
     nn <- length(x)
     if(cmax) {
@@ -590,6 +723,7 @@ function(x, threshold, start, npp = length(x), cmax = FALSE, r = 1, ulow = -Inf,
     if(warn.inf && do.call("nllh", start.arg) == 1e6)
         warning("negative log-likelihood is infinite at starting values")
     opt <- optim(start, nllh, hessian = TRUE, ..., method = method)
+	if(is.null(names(opt$par))) names(opt$par) <- nm
     if (opt$convergence != 0) {
         warning("optimization may not have succeeded")
         if(opt$convergence == 1) opt$convergence <- "iteration limit reached"
@@ -621,7 +755,7 @@ function(x, threshold, start, npp = length(x), cmax = FALSE, r = 1, ulow = -Inf,
     list(estimate = opt$par, std.err = std.err, fixed = unlist(fixed.param),
         param = param, deviance = 2*opt$value, corr = corr, var.cov = var.cov,
         convergence = opt$convergence, counts = opt$counts, message = opt$message,
-        threshold = threshold, r = r, ulow = ulow, rlow = rlow, npp = npp,
+        threshold = threshold, cmax = cmax, r = r, ulow = ulow, rlow = rlow, npp = npp,
         nhigh = nhigh, nat = nat, pat = pat, extind = extind,
         data = x, exceedances = exceed, mper = mper, scale = scale)
 }
